@@ -57,16 +57,15 @@ class Sandbox(object):
                 cls.__init__(self, **kwargs)
 
         # Logging options
-        if self.options.singlestep:
-            self.jitter.jit.log_mn = True
-            self.jitter.jit.log_regs = True
+        self.jitter.set_trace_log(
+            trace_instr=self.options.singlestep,
+            trace_regs=self.options.singlestep,
+            trace_new_blocks=self.options.dumpblocs
+        )
 
         if not self.options.quiet_function_calls:
             log_func.setLevel(logging.INFO)
 
-        if self.options.dumpblocs:
-            self.jitter.jit.log_newbloc = True
-
     @classmethod
     def parser(cls, *args, **kwargs):
         """
@@ -237,7 +236,7 @@ class OS_Win(OS):
         self.jitter.add_lib_handler(libs, methods)
 
         # Manage SEH
-        if self.options.use_seh:
+        if self.options.use_windows_structs:
             win_api_x86_32_seh.main_pe_name = fname_basename
             win_api_x86_32_seh.main_pe = self.pe
             win_api_x86_32.winobjs.hcurmodule = self.pe.NThdr.ImageBase
@@ -252,8 +251,8 @@ class OS_Win(OS):
     def update_parser(cls, parser):
         parser.add_argument('-o', "--load-hdr", action="store_true",
                             help="Load pe hdr")
-        parser.add_argument('-y', "--use-seh", action="store_true",
-                            help="Use windows SEH")
+        parser.add_argument('-y', "--use-windows-structs", action="store_true",
+                            help="Create and use windows structures (peb, ldr, seh, ...)")
         parser.add_argument('-l', "--loadbasedll", action="store_true",
                             help="Load base dll (path './win_dll')")
         parser.add_argument('-r', "--parse-resources",
