@@ -36,6 +36,9 @@ class CTypeBase(object):
     def _typerepr(self):
         return self.__repr
 
+    def __eq__(self, other):
+        raise NotImplementedError("Abstract method")
+
     def __ne__(self, other):
         return not self.__eq__(other)
 
@@ -58,7 +61,7 @@ class CTypeId(CTypeBase):
 
     def __init__(self, *names):
         # Type specifier order does not matter
-        # so the cannonical form is ordered
+        # so the canonical form is ordered
         self.names = tuple(sorted(names))
         super(CTypeId, self).__init__()
 
@@ -69,6 +72,9 @@ class CTypeId(CTypeBase):
         return (self.eq_base(other) and
                 self.names == other.names)
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __str__(self):
         return "<Id:%s>" % ', '.join(self.names)
 
@@ -92,6 +98,9 @@ class CTypeArray(CTypeBase):
                 self.target == other.target and
                 self.size == other.size)
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __str__(self):
         return "<Array[%s]:%s>" % (self.size, str(self.target))
 
@@ -113,6 +122,9 @@ class CTypePtr(CTypeBase):
         return (self.eq_base(other) and
                 self.target == other.target)
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __str__(self):
         return "<Ptr:%s>" % str(self.target)
 
@@ -139,6 +151,9 @@ class CTypeStruct(CTypeBase):
                 self.name == other.name and
                 self.fields == other.fields)
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __str__(self):
         out = []
         out.append("<Struct:%s>" % self.name)
@@ -191,6 +206,9 @@ class CTypeEnum(CTypeBase):
         return (self.eq_base(other) and
                 self.name == other.name)
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __str__(self):
         return "<Enum:%s>" % self.name
 
@@ -224,6 +242,9 @@ class CTypeFunc(CTypeBase):
                 self.type_ret == other.type_ret and
                 self.args == other.args)
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __str__(self):
         return "<Func:%s (%s) %s(%s)>" % (self.type_ret,
                                           self.abi,
@@ -240,6 +261,9 @@ class CTypeEllipsis(CTypeBase):
     def __eq__(self, other):
         return self.eq_base(other)
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __str__(self):
         return "<Ellipsis>"
 
@@ -258,6 +282,9 @@ class CTypeSizeof(CTypeBase):
         return (self.eq_base(other) and
                 self.target == other.target)
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __str__(self):
         return "<Sizeof(%s)>" % self.target
 
@@ -345,7 +372,7 @@ class CAstTypes(object):
         }
 
     def gen_uniq_name(self):
-        """Generate uniq name for unamed strucs/union"""
+        """Generate uniq name for unnamed strucs/union"""
         cpt = self.cpt
         self.cpt += 1
         return self.INTERNAL_PREFIX + "%d" % cpt
@@ -527,7 +554,7 @@ class CAstTypes(object):
         elif isinstance(ast, c_ast.Constant):
             result = int(ast.value, 0)
         elif isinstance(ast, c_ast.Cast):
-            # TODO: Can trunc intergers?
+            # TODO: Can trunc integers?
             result = self.ast_eval_int(ast.expr)
         else:
             raise NotImplementedError("Not implemented!")
