@@ -267,6 +267,12 @@ class DiGraph(object):
             for next_node in next_cb(node):
                 todo.add(next_node)
 
+    def predecessors_stop_node_iter(self, node, head):
+        if node == head:
+            raise StopIteration
+        for next_node in self.predecessors_iter(node):
+            yield next_node
+
     def reachable_sons(self, head):
         """Compute all nodes reachable from node @head. Each son is an
         immediate successor of an arbitrary, already yielded son of @head"""
@@ -277,14 +283,26 @@ class DiGraph(object):
         predecessor of an arbitrary, already yielded parent of @leaf"""
         return self._reachable_nodes(leaf, self.predecessors_iter)
 
+    def reachable_parents_stop_node(self, leaf, head):
+        """Compute all parents of node @leaf. Each parent is an immediate
+        predecessor of an arbitrary, already yielded parent of @leaf.
+        Do not compute reachables past @head node"""
+        return self._reachable_nodes(
+            leaf,
+            lambda node_cur: self.predecessors_stop_node_iter(
+                node_cur, head
+            )
+        )
+
+
     @staticmethod
     def _compute_generic_dominators(head, reachable_cb, prev_cb, next_cb):
         """Generic algorithm to compute either the dominators or postdominators
         of the graph.
         @head: the head/leaf of the graph
         @reachable_cb: sons/parents of the head/leaf
-        @prev_cb: return predecessors/succesors of a node
-        @next_cb: return succesors/predecessors of a node
+        @prev_cb: return predecessors/successors of a node
+        @next_cb: return successors/predecessors of a node
         """
 
         nodes = set(reachable_cb(head))
@@ -339,6 +357,22 @@ class DiGraph(object):
                                                 self.successors_iter,
                                                 self.predecessors_iter)
 
+
+
+
+    def compute_dominator_tree(self, head):
+        """
+        Computes the dominator tree of a graph
+        :param head: head of graph
+        :return: DiGraph
+        """
+        idoms = self.compute_immediate_dominators(head)
+        dominator_tree = DiGraph()
+        for node in idoms:
+            dominator_tree.add_edge(idoms[node], node)
+
+        return dominator_tree
+
     @staticmethod
     def _walk_generic_dominator(node, gen_dominators, succ_cb):
         """Generic algorithm to return an iterator of the ordered list of
@@ -348,7 +382,7 @@ class DiGraph(object):
         @node: The start node
         @gen_dominators: The dictionary containing at least node's
         dominators/post_dominators
-        @succ_cb: return predecessors/succesors of a node
+        @succ_cb: return predecessors/successors of a node
 
         """
         # Init
@@ -422,6 +456,18 @@ class DiGraph(object):
                     break
         return idoms
 
+    def compute_immediate_postdominators(self,tail):
+        """Compute the immediate postdominators of the graph"""
+        postdominators = self.compute_postdominators(tail)
+        ipdoms = {}
+
+        for node in postdominators:
+            for successor in self.walk_postdominators(node, postdominators):
+                if successor in postdominators[node] and node != successor:
+                    ipdoms[node] = successor
+                    break
+        return ipdoms
+
     def compute_dominance_frontier(self, head):
         """
         Compute the dominance frontier of the graph
@@ -710,7 +756,7 @@ class MatchGraphJoker(object):
 
     def __init__(self, restrict_in=True, restrict_out=True, filt=None,
                  name=None):
-        """Instanciate a MatchGraphJoker, with restrictions
+        """Instantiate a MatchGraphJoker, with restrictions
         @restrict_in: (optional) if set, the number of predecessors of the
                       matched node must be the same than the joker node in the
                       associated MatchGraph
@@ -763,7 +809,7 @@ class MatchGraph(DiGraph):
     restrictions.
     The implemented algorithm is a naive approach.
 
-    The recommended way to instanciate a MatchGraph is the use of
+    The recommended way to instantiate a MatchGraph is the use of
     MatchGraphJoker.
     """
 
@@ -891,7 +937,7 @@ class MatchGraph(DiGraph):
         # Partial solution: nodes corrects, edges between these nodes corrects
         # A partial solution is a dictionary MatchGraphJoker -> @graph's node
         todo = list()  # Dictionnaries containing partial solution
-        done = list()  # Aleady computed partial solutions
+        done = list()  # Already computed partial solutions
 
         # Elect first candidates
         to_match = next(iter(self._nodes))
