@@ -91,7 +91,7 @@ And access the fields:
 MemUnion and MemBitField can also be subclassed, the `fields` field being
 in the format expected by, respectively, Union and BitField.
 
-The `addr` argument can be omited if an allocator is set, in which case the
+The `addr` argument can be omitted if an allocator is set, in which case the
 structure will be automatically allocated in memory:
 
     my_heap = miasm2.os_dep.common.heap()
@@ -277,7 +277,7 @@ class Type(object):
         """Set a VmMngr memory from a value.
 
         @vm: VmMngr instance
-        @addr: the start adress in memory to set
+        @addr: the start address in memory to set
         @val: the python value to serialize in @vm at @addr
         """
         raw = self._pack(val)
@@ -293,7 +293,7 @@ class Type(object):
         """Returns a class with a (vm, addr) constructor that allows to
         interact with this type in memory.
 
-        In compilation terms, it returns a class allowing to instanciate an
+        In compilation terms, it returns a class allowing to instantiate an
         lvalue of this type.
 
         @return: a MemType subclass.
@@ -305,7 +305,7 @@ class Type(object):
         return pinned_type
 
     def _build_pinned_type(self):
-        """Builds the MemType subclass allowing to interract with this type.
+        """Builds the MemType subclass allowing to interact with this type.
 
         Called by self.lval when it is not in cache.
         """
@@ -340,6 +340,9 @@ class Type(object):
     def __neq__(self, other):
         return not self == other
 
+    def __eq__(self, other):
+        raise NotImplementedError("Abstract method")
+
     def __ne__(self, other):
         return not self.__eq__(other)
 
@@ -369,6 +372,9 @@ class RawStruct(Type):
     def __eq__(self, other):
         return self.__class__ == other.__class__ and self._fmt == other._fmt
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __hash__(self):
         return hash((self.__class__, self._fmt))
 
@@ -413,7 +419,7 @@ class Ptr(Num):
             If a Type is given, it is transformed into a MemType with
             TheType.lval.
         *type_args, **type_kwargs: arguments to pass to the the pointed
-            MemType when instanciating it (e.g. for MemStr encoding or
+            MemType when instantiating it (e.g. for MemStr encoding or
             MemArray field_type).
         """
         if (not isinstance(dst_type, Type) and
@@ -484,7 +490,7 @@ class Ptr(Num):
         """
         # Sanity check
         if self.dst_type != val.get_type():
-            log.warning("Original type was %s, overriden by value of type %s",
+            log.warning("Original type was %s, overridden by value of type %s",
                         self._dst_type.__name__, val.__class__.__name__)
 
         # Actual job
@@ -503,6 +509,9 @@ class Ptr(Num):
                 self._type_args == other._type_args and \
                 self._type_kwargs == other._type_kwargs
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __hash__(self):
         return hash((super(Ptr, self).__hash__(), self.dst_type,
             self._type_args))
@@ -673,6 +682,9 @@ class Struct(Type):
                 self.fields == other.fields and \
                 self.name == other.name
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __hash__(self):
         # Only hash name, not fields, because if a field is a Ptr to this
         # Struct type, an infinite loop occurs
@@ -862,6 +874,9 @@ class Array(Type):
                 self.field_type == other.field_type and \
                 self.array_len == other.array_len
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __hash__(self):
         return hash((self.__class__, self.field_type, self.array_len))
 
@@ -871,7 +886,7 @@ class Bits(Type):
     bits of a Num.
 
     The @backing_num is used to know how to serialize/deserialize data in vm,
-    but getting/setting this fields only affects bits from @bit_offset to
+    but getting/setting this fields only assign bits from @bit_offset to
     @bit_offset + @bits. Masking and shifting is handled by the class, the aim
     is to provide a transparent way to set and get some bits of a num.
     """
@@ -926,6 +941,9 @@ class Bits(Type):
                 self._num == other._num and self._bits == other._bits and \
                 self._bit_offset == other._bit_offset
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __hash__(self):
         return hash((self.__class__, self._num, self._bits, self._bit_offset))
 
@@ -962,7 +980,7 @@ class BitField(Union):
     """
 
     def __init__(self, backing_num, bit_list):
-        """@backing num: Num intance, @bit_list: [(name, n_bits)]"""
+        """@backing num: Num instance, @bit_list: [(name, n_bits)]"""
         self._num = backing_num
         fields = []
         offset = 0
@@ -983,6 +1001,9 @@ class BitField(Union):
         return self.__class__ == other.__class__ and \
                 self._num == other._num and super(BitField, self).__eq__(other)
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __hash__(self):
         return hash((super(BitField, self).__hash__(), self._num))
 
@@ -1095,6 +1116,9 @@ class Str(Type):
     def __eq__(self, other):
         return self.__class__ == other.__class__ and self._enc == other._enc
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __hash__(self):
         return hash((self.__class__, self._enc))
 
@@ -1111,6 +1135,9 @@ class Void(Type):
     def __eq__(self, other):
         return self.__class__ == other.__class__
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __hash__(self):
         return hash(self.__class__)
 
@@ -1134,7 +1161,7 @@ class Self(Void):
     def _build_pinned_type(self):
         return MemSelf
 
-# To avoid reinstanciation when testing equality
+# To avoid reinstantiation when testing equality
 SELF_TYPE_INSTANCE = Self()
 VOID_TYPE_INSTANCE = Void()
 
@@ -1205,7 +1232,7 @@ class MemType(object):
 
     @classmethod
     def set_allocator(cls, alloc_func):
-        """Set an allocator for this class; allows to instanciate statically
+        """Set an allocator for this class; allows to instantiate statically
         sized MemTypes (i.e. sizeof() is implemented) without specifying the
         address (the object is allocated by @alloc_func in the vm).
 
@@ -1330,7 +1357,7 @@ class MemStruct(MemType):
     The mechanism is the following:
         - set a "fields" class field to be a list of
           (<field_name (str)>, <Type_subclass_instance>)
-        - instances of this class will have properties to interract with these
+        - instances of this class will have properties to interact with these
           fields.
 
     Example:
