@@ -3,9 +3,8 @@ import logging
 from collections import MutableMapping
 
 from miasm2.expression.expression import ExprOp, ExprId, ExprLoc, ExprInt, \
-    ExprMem, ExprCompose, ExprSlice, ExprCond, ExprAff
-from miasm2.expression.simplifications import expr_simp
-from miasm2.core import asmblock
+    ExprMem, ExprCompose, ExprSlice, ExprCond
+from miasm2.expression.simplifications import expr_simp_explicit
 from miasm2.ir.ir import AssignBlock
 
 log = logging.getLogger("symbexec")
@@ -15,16 +14,16 @@ log.addHandler(console_handler)
 log.setLevel(logging.INFO)
 
 
-def get_block(ir_arch, mdis, addr):
+def get_block(ir_arch, ircfg, mdis, addr):
     """Get IRBlock at address @addr"""
-    lbl = ir_arch.get_loc_key(addr)
-    if not lbl in ir_arch.blocks:
-        offset = mdis.symbol_pool.loc_key_to_offset(lbl)
+    loc_key = ircfg.get_or_create_loc_key(addr)
+    if not loc_key in ircfg.blocks:
+        offset = mdis.loc_db.get_location_offset(loc_key)
         block = mdis.dis_block(offset)
-        ir_arch.add_block(block)
-    irblock = ir_arch.get_block(lbl)
+        ir_arch.add_asmblock_to_ircfg(block, ircfg)
+    irblock = ircfg.get_block(loc_key)
     if irblock is None:
-        raise LookupError('No block found at that address: %s' % lbl)
+        raise LookupError('No block found at that address: %s' % ir_arch.loc_db.pretty_str(loc_key))
     return irblock
 
 
@@ -82,7 +81,7 @@ class SymbolicState(StateEngine):
 
     @property
     def symbols(self):
-        """Return the dictionnary of known symbols"""
+        """Return the dictionary of known symbols"""
         return dict(self._symbols)
 
 
@@ -120,7 +119,7 @@ class MemArray(MutableMapping):
 
     The value associated to a given offset is a description of the slice of a
     stored expression. The slice size depends on the configutation of the
-    MemArray. For example, for a slice size of 8 bits, the affectation:
+    MemArray. For example, for a slice size of 8 bits, the assignment:
     - @32[EAX+0x10] = EBX
 
     will store for the base EAX:
@@ -139,7 +138,7 @@ class MemArray(MutableMapping):
 
     """
 
-    def __init__(self, base, expr_simp=expr_simp):
+    def __init__(self, base, expr_simp=expr_simp_explicit):
         self._base = base
         self.expr_simp = expr_simp
         self._mask = int(base.mask)
@@ -199,7 +198,7 @@ class MemArray(MutableMapping):
             ptr = base
         else:
             ptr = base + ExprInt(offset, base.size)
-        return ptr
+        return ptr.canonize()
 
     def read(self, offset, size):
         """
@@ -257,8 +256,8 @@ class MemArray(MutableMapping):
                 continue
             if data_a.is_mem() and data_b.is_mem():
                 # Read consecutive bytes of a memory variable
-                ptr_base_a, ptr_offset_a = get_expr_base_offset(data_a.arg)
-                ptr_base_b, ptr_offset_b = get_expr_base_offset(data_b.arg)
+                ptr_base_a, ptr_offset_a = get_expr_base_offset(data_a.ptr)
+                ptr_base_b, ptr_offset_b = get_expr_base_offset(data_b.ptr)
                 if ptr_base_a != ptr_base_b:
                     index += 1
                     continue
@@ -304,11 +303,11 @@ class MemArray(MutableMapping):
             # Special case: Simplify slice of pointer (simplification is ok
             # here, as we won't store the simplified expression)
             if tmp.is_slice() and tmp.arg.is_mem() and tmp.start % 8 == 0:
-                new_ptr = self.expr_simp(tmp.arg.arg + ExprInt(tmp.start / 8, tmp.arg.arg.size))
+                new_ptr = self.expr_simp(tmp.arg.ptr + ExprInt(tmp.start / 8, tmp.arg.ptr.size))
                 tmp = ExprMem(new_ptr, tmp.stop - tmp.start)
             # Test if write to original value
             if tmp.is_mem():
-                src_ptr, src_off = get_expr_base_offset(tmp.arg)
+                src_ptr, src_off = get_expr_base_offset(tmp.ptr)
                 if src_ptr == self.base and src_off == request_offset:
                     del self._offset_to_expr[request_offset]
 
@@ -462,7 +461,7 @@ class MemSparse(object):
 
     """
 
-    def __init__(self, addrsize, expr_simp=expr_simp):
+    def __init__(self, addrsize, expr_simp=expr_simp_explicit):
         """
         @addrsize: size (in bits) of the addresses manipulated by the MemSparse
         @expr_simp: an ExpressionSimplifier instance
@@ -478,7 +477,7 @@ class MemSparse(object):
         """
         if not expr.is_mem():
             return False
-        ptr = expr.arg
+        ptr = expr.ptr
         base, offset = get_expr_base_offset(ptr)
         memarray = self.base_to_memarray.get(base, None)
         if memarray is None:
@@ -494,7 +493,7 @@ class MemSparse(object):
         """
         if not expr.is_mem():
             return False
-        ptr = expr.arg
+        ptr = expr.ptr
         base, offset = get_expr_base_offset(ptr)
         memarray = self.base_to_memarray.get(base, None)
         if memarray is None:
@@ -522,7 +521,7 @@ class MemSparse(object):
         Delete a value @expr *fully* present in memory
         For partial delete, use delete_partial
         """
-        ptr = expr.arg
+        ptr = expr.ptr
         base, offset = get_expr_base_offset(ptr)
         memarray = self.base_to_memarray.get(base, None)
         if memarray is None:
@@ -539,7 +538,7 @@ class MemSparse(object):
         Delete @expr from memory. Skip parts of @expr which are not present in
         memory.
         """
-        ptr = expr.arg
+        ptr = expr.ptr
         base, offset = get_expr_base_offset(ptr)
         memarray = self.base_to_memarray.get(base, None)
         if memarray is None:
@@ -605,7 +604,7 @@ class MemSparse(object):
 class SymbolMngr(object):
     """Symbolic store manager (IDs and MEMs)"""
 
-    def __init__(self, init=None, addrsize=None, expr_simp=expr_simp):
+    def __init__(self, init=None, addrsize=None, expr_simp=expr_simp_explicit):
         assert addrsize is not None
         if init is None:
             init = {}
@@ -658,7 +657,7 @@ class SymbolMngr(object):
         elif src.is_mem():
             # Only byte aligned accesses are supported for now
             assert src.size % 8 == 0
-            return self.symbols_mem.read(src.arg, src.size)
+            return self.symbols_mem.read(src.ptr, src.size)
         else:
             raise TypeError("Bad source expr")
 
@@ -678,7 +677,7 @@ class SymbolMngr(object):
         elif dst.is_mem():
             # Only byte aligned accesses are supported for now
             assert dst.size % 8 == 0
-            self.symbols_mem.write(dst.arg, src)
+            self.symbols_mem.write(dst.ptr, src)
         else:
             raise TypeError("Bad destination expr")
 
@@ -805,10 +804,10 @@ class SymbolicExecutionEngine(object):
 
     StateEngine = SymbolicState
 
-    def __init__(self, ir_arch, state,
+    def __init__(self, ir_arch, state=None,
                  func_read=None,
                  func_write=None,
-                 sb_expr_simp=expr_simp):
+                 sb_expr_simp=expr_simp_explicit):
 
         self.expr_to_visitor = {
             ExprInt: self.eval_exprint,
@@ -821,7 +820,10 @@ class SymbolicExecutionEngine(object):
             ExprCompose: self.eval_exprcompose,
         }
 
-        self.symbols = SymbolMngr(addrsize=ir_arch.addrsize, expr_simp=expr_simp)
+        if state is None:
+            state = {}
+
+        self.symbols = SymbolMngr(addrsize=ir_arch.addrsize, expr_simp=sb_expr_simp)
 
         for dst, src in state.iteritems():
             self.symbols.write(dst, src)
@@ -892,7 +894,7 @@ class SymbolicExecutionEngine(object):
 
     def eval_exprloc(self, expr, **kwargs):
         """[DEV]: Evaluate an ExprLoc using the current state"""
-        offset = self.ir_arch.symbol_pool.loc_key_to_offset(expr.loc_key)
+        offset = self.ir_arch.loc_db.get_location_offset(expr.loc_key)
         if offset is not None:
             ret = ExprInt(offset, expr.size)
         else:
@@ -904,7 +906,7 @@ class SymbolicExecutionEngine(object):
         This function first evaluate the memory pointer value.
         Override 'mem_read' to modify the effective memory accesses
         """
-        ptr = self.eval_expr_visitor(expr.arg, **kwargs)
+        ptr = self.eval_expr_visitor(expr.ptr, **kwargs)
         mem = ExprMem(ptr, expr.size)
         ret = self.mem_read(mem)
         return ret
@@ -943,7 +945,7 @@ class SymbolicExecutionEngine(object):
     def eval_expr(self, expr, eval_cache=None):
         """
         Evaluate @expr
-        @expr: Expresion instance to evaluate
+        @expr: Expression instance to evaluate
         @cache: None or dictionary linking variables to their values
         """
         if eval_cache is None:
@@ -961,7 +963,7 @@ class SymbolicExecutionEngine(object):
         @mems: track mems only
         """
         if init_state is None:
-            init_state = self.ir_arch.arch.regs.regs_init
+            init_state = {}
         if ids:
             for variable, value in self.symbols.symbols_id.iteritems():
                 if variable in init_state and init_state[variable] == value:
@@ -996,7 +998,7 @@ class SymbolicExecutionEngine(object):
         for dst, src in assignblk.iteritems():
             src = self.eval_expr(src, eval_cache)
             if dst.is_mem():
-                ptr = self.eval_expr(dst.arg, eval_cache)
+                ptr = self.eval_expr(dst.ptr, eval_cache)
                 # Test if mem lookup is known
                 tmp = ExprMem(ptr, dst.size)
                 pool_out[tmp] = src
@@ -1052,18 +1054,18 @@ class SymbolicExecutionEngine(object):
 
         return dst
 
-    def run_block_at(self, addr, step=False):
+    def run_block_at(self, ircfg, addr, step=False):
         """
         Symbolic execution of the block at @addr
         @addr: address to execute (int or ExprInt or label)
         @step: display intermediate steps
         """
-        irblock = self.ir_arch.get_block(addr)
+        irblock = ircfg.get_block(addr)
         if irblock is not None:
             addr = self.eval_updt_irblock(irblock, step=step)
         return addr
 
-    def run_at(self, addr, lbl_stop=None, step=False):
+    def run_at(self, ircfg, addr, lbl_stop=None, step=False):
         """
         Symbolic execution starting at @addr
         @addr: address to execute (int or ExprInt or label)
@@ -1071,7 +1073,7 @@ class SymbolicExecutionEngine(object):
         @step: display intermediate steps
         """
         while True:
-            irblock = self.ir_arch.get_block(addr)
+            irblock = ircfg.get_block(addr)
             if irblock is None:
                 break
             if irblock.loc_key == lbl_stop:
@@ -1122,7 +1124,7 @@ class SymbolicExecutionEngine(object):
 
         # Extract known parts in symbols
         assert expr.size % 8 == 0
-        ptr = expr.arg
+        ptr = expr.ptr
         known = []
         ptrs = []
         for index in xrange(expr.size / 8):
@@ -1268,9 +1270,9 @@ class symbexec(SymbolicExecutionEngine):
     def __init__(self, ir_arch, known_symbols,
                  func_read=None,
                  func_write=None,
-                 sb_expr_simp=expr_simp):
+                 sb_expr_simp=expr_simp_explicit):
         warnings.warn("Deprecated API: use SymbolicExecutionEngine")
         super(symbexec, self).__init__(ir_arch, known_symbols,
                                        func_read,
                                        func_write,
-                                       sb_expr_simp=expr_simp)
+                                       sb_expr_simp=sb_expr_simp)
