@@ -1,7 +1,9 @@
 #ifndef VM_MNGR_PY_H
 #define VM_MNGR_PY_H
 
-
+#ifdef _WIN32
+#define SIGALRM 0
+#endif
 
 typedef struct {
 	PyObject_HEAD
