@@ -24,7 +24,7 @@ import struct
 from elfesteem import pe_init
 
 from miasm2.jitter.csts import PAGE_READ, PAGE_WRITE
-from miasm2.core.utils import pck32, upck32
+from miasm2.core.utils import pck32
 import miasm2.arch.x86.regs as x86_regs
 
 from miasm2.os_dep.win_32_structs import LdrDataEntry, ListEntry, \
@@ -85,7 +85,7 @@ MAX_SEH = 5
 
 def build_teb(jitter, teb_address):
     """
-    Build TEB informations using following structure:
+    Build TEB information using following structure:
 
     @jitter: jitter instance
     @teb_address: the TEB address
@@ -111,7 +111,7 @@ def build_teb(jitter, teb_address):
 
 def build_peb(jitter, peb_address):
     """
-    Build PEB informations using following structure:
+    Build PEB information using following structure:
 
     @jitter: jitter instance
     @peb_address: the PEB address
@@ -135,7 +135,7 @@ def build_peb(jitter, peb_address):
 
 def build_ldr_data(jitter, modules_info):
     """
-    Build Loader informations using following structure:
+    Build Loader information using following structure:
 
     +0x000 Length                          : Uint4B
     +0x004 Initialized                     : UChar
@@ -232,7 +232,7 @@ def create_modules_chain(jitter, name2module):
     out = ""
     for i, (fname, pe_obj) in enumerate(name2module.items(), 1):
         if pe_obj is None:
-            log.warning("Unknown module: ommited from link list (%r)",
+            log.warning("Unknown module: omitted from link list (%r)",
                         fname)
             continue
         addr = base_addr + i * 0x1000
@@ -368,7 +368,7 @@ def fix_InInitializationOrderModuleList(jitter, modules_info):
 
 def add_process_env(jitter):
     """
-    Build a process environement structure
+    Build a process environment structure
     @jitter: jitter instance
     """
 
@@ -614,8 +614,8 @@ def return_from_seh(jitter):
     @jitter: jitter instance"""
 
     # Get object addresses
-    seh_address = upck32(jitter.vm.get_mem(jitter.cpu.ESP + 0x4, 4))
-    context_address = upck32(jitter.vm.get_mem(jitter.cpu.ESP + 0x8, 4))
+    seh_address = jitter.vm.get_u32(jitter.cpu.ESP + 0x4)
+    context_address = jitter.vm.get_u32(jitter.cpu.ESP + 0x8)
 
     # Get registers changes
     log.info('Context address: %x', context_address)
